/**
 ******************************************************************************
 * File Name          : main.c
 * Description        : Main program body
 ******************************************************************************
 *
 * COPYRIGHT(c) 2016 STMicroelectronics
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of STMicroelectronics nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 */


/* Includes ------------------------------------------------------------------*/
#include "stm32f7xx_hal.h"

/* USER CODE BEGIN Includes */
#include "main.h"
#include "Screen1DLG.h"
#include "KNOB.h"
#ifdef STEP_1
#include "WM.h"
#endif
#ifdef STEP_2
#include "DIALOG.h"
#endif

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* Private variables ---------------------------------------------------------*/
/*debug var start*/
uint8_t test_buffer[];
/*debug var end*/
TIM_HandleTypeDef TimHandle;
#ifdef STEP_1
uint8_t GUI_Initialized = 0; //check if the STemWin is initialized (necessary for touch support)
#endif

/*********************************************************************
 *
 *       Dialog resources
 *
 **********************************************************************
 */

#ifdef STEP_2
static WM_HWIN _hDlgScreen1; /*the dialog box handle*/
#endif

#ifdef STEP_10
static uint8_t hour = 0, minute = 0; /*variable contai hour and minute time*/
static uint32_t miliceconds = 0; /*milisecond time*/
#endif
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);
static void MX_GPIO_Init(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/
#ifdef STEP_3
void
TouchTimer_Init(void); /*Initialize timer for touch sensor*/
void
BSP_Pointer_Update(void); /*function for the pointer update*/
#endif
#ifdef STEP_2
extern WM_HWIN CreateScreen1(void);
extern void UpdateTimeScreen1(WM_HWIN aDlgScreen, uint8_t aHour, uint8_t aMinute);
#endif
/* USER CODE END PFP */

/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

int main(void)
{

	/* USER CODE BEGIN 1 */

	/* USER CODE END 1 */

	/* MCU Configuration----------------------------------------------------------*/

	/* Reset of all peripherals, Initializes the Flash interface and the Systick. */
	HAL_Init();

	/* Configure the system clock */
	SystemClock_Config();

	/* Initialize all configured peripherals */
	MX_GPIO_Init();

	/* USER CODE BEGIN 2 */
#ifdef STEP_3
	TouchTimer_Init();/*Initialize the Touch and enable the Timer for touch periodical reading*/
#endif

#ifdef STEP_1
	BSP_SDRAM_Init(); /* Initializes the SDRAM device */
	__HAL_RCC_CRC_CLK_ENABLE()
	; /* Enable the CRC Module */
	GUI_Init();/*Initialize and start STemWin*/
	GUI_Initialized = 1; /*STemWin initialize allow the pointer update from Touch */
    GUI_DispStringAt("Starting...", 0, 0);/*set string on display*/
	GUI_Exec();/*show last changes on the screen*/

#endif

#ifdef STEP_11
	WM_MULTIBUF_Enable(1); /*enable double buffer for window manager*/
#endif

#ifdef STEP_2
	_hDlgScreen1 = CreateScreen1(); //create dialog with all window items
#endif
	/* USER CODE END 2 */

	/* Infinite loop */
	/* USER CODE BEGIN WHILE */
	while (1)
	{
		/* USER CODE END WHILE */
		/* USER CODE BEGIN 3 */
		GUI_Exec();/*show last changes on the screen*/
		HAL_Delay(10);

	}
	/* USER CODE END 3 */

}

/** System Clock Configuration
 */
void SystemClock_Config(void)
{

	RCC_OscInitTypeDef RCC_OscInitStruct;
	RCC_ClkInitTypeDef RCC_ClkInitStruct;

	__HAL_RCC_PWR_CLK_ENABLE()
	;

	__HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = 16;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLM = 8;
    RCC_OscInitStruct.PLL.PLLN = 200;
	RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
	RCC_OscInitStruct.PLL.PLLQ = 2;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
	{
		Error_Handler();
	}

	if (HAL_PWREx_EnableOverDrive() != HAL_OK)
	{
		Error_Handler();
	}

	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK
			| RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;
  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_6) != HAL_OK)
	{
		Error_Handler();
	}

	HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq() / 1000);

	HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

	/* SysTick_IRQn interrupt configuration */
	HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/** Configure pins as 
 * Analog
 * Input
 * Output
 * EVENT_OUT
 * EXTI
 */
static void MX_GPIO_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStruct;

	/* GPIO Ports Clock Enable */
    __HAL_RCC_GPIOH_CLK_ENABLE();
    __HAL_RCC_GPIOI_CLK_ENABLE();

    __GPIOI_CLK_ENABLE();

    GPIO_InitStruct.Pin = GPIO_PIN_1;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
    HAL_GPIO_Init(GPIOI, &GPIO_InitStruct);



}

/* USER CODE BEGIN 4 */
#ifdef STEP_3
/*function initialize timer which will periodically read the touch controller state in interrupt routine*/
void TouchTimer_Init()
{
	uint32_t uwPrescalerValue = 0;
	/* Compute the prescaler value to have TIM3 counter clock equal to 10 KHz */
	uwPrescalerValue = (uint32_t) ((SystemCoreClock / 2) / 10000) - 1;

	BSP_TS_Init(480, 272);/*Initialise the TS controller on screen resolution*/

	/* Set TIMx instance */
	TimHandle.Instance = TIM3;

	/* Initialise TIM3 peripheral as follows:
	 + Period = 500 - 1
	 + Prescaler = ((SystemCoreClock/2)/10000) - 1
	 + ClockDivision = 0
	 + Counter direction = Up
	 */
	TimHandle.Init.Period = 500 - 1;
	TimHandle.Init.Prescaler = uwPrescalerValue;
	TimHandle.Init.ClockDivision = 0;
	TimHandle.Init.CounterMode = TIM_COUNTERMODE_UP;
	if (HAL_TIM_Base_Init(&TimHandle) != HAL_OK)
	{
		while (1)
		{
		}
	}

	/*##-2- Start the TIM Base generation in interrupt mode ####################*/
	/* Start Channel1 */
	if (HAL_TIM_Base_Start_IT(&TimHandle) != HAL_OK)
	{
		while (1)
		{
		}
	}
}

/*the timer interrupt callback*/
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
	/*check if the STemWin is initialize before use STemWin API*/
	if (GUI_Initialized == 1)
	{
		BSP_Pointer_Update();/*handle the touch changes*/
	}
}

/**
 * @brief  Provide the GUI with current state of the touch screen
 * @param  None
 * @retval None
 */
#ifdef __ICCARM__
#pragma optimize=none
#endif
void BSP_Pointer_Update(void)
{
	GUI_PID_STATE TS_State;/*structure which reports the touch state to STemWin*/
	static TS_StateTypeDef prev_state;/*previous touch state from the touch sensor used from BSP package*/
	TS_StateTypeDef ts;/*actual touch state from the touch sensor used from BSP package*/
	uint16_t xDiff, yDiff;/*difference in postitions between touch states*/

	BSP_TS_GetState(&ts);/*read the touch state from touch sensor (BSP API)*/

	TS_State.Pressed = ts.touchDetected; /*store pressed state to STemWin structure*/

	xDiff = (prev_state.touchX[0] > ts.touchX[0]) ?
			(prev_state.touchX[0] - ts.touchX[0]) :
			(ts.touchX[0] - prev_state.touchX[0]);/*calculate x difference*/
	yDiff = (prev_state.touchY[0] > ts.touchY[0]) ?
			(prev_state.touchY[0] - ts.touchY[0]) :
			(ts.touchY[0] - prev_state.touchY[0]);/*calculate y difference*/

	/*check if the touch is pressed*/
	if ((prev_state.touchDetected != ts.touchDetected) || (xDiff > 3)
			|| (yDiff > 3))
	{
		prev_state.touchDetected = ts.touchDetected;

		/*check touch movings*/
		if ((ts.touchX[0] != 0) && (ts.touchY[0] != 0))
		{
			prev_state.touchX[0] = ts.touchX[0];
			prev_state.touchY[0] = ts.touchY[0];
		}

		TS_State.Layer = 0;
		TS_State.x = prev_state.touchX[0];
		TS_State.y = prev_state.touchY[0];

		GUI_TOUCH_StoreStateEx(&TS_State);/*send touch state to STemWin*/
	}
}
#endif

#ifdef STEP_10
/*the systick callback from hal called each 1ms
 * here we can implement the clock increment
 * */
void HAL_SYSTICK_Callback(void)
{
	miliceconds++;/*increment millisecond variable*/
	/*check if we reach maximum value (1s)*/
	if (miliceconds >= (1000))
	{
		miliceconds = 0;/*reset the millisecond counter*/
		minute++;/*increment minute counter*/
		/*check if we reach 60minutes*/
		if (minute >= 60)
		{
			minute = 0;/*reset minute counter*/
			hour++;/*increment hour counter*/
			/*check if we reached 24 hour*/
			if (hour >= 24)
			{
				hour = 0;/*reset hour counter*/
			}
		}
		/*check if the STemWin is initalized*/
		if (GUI_Initialized)
		{
			UpdateTimeScreen1(_hDlgScreen1,hour,minute);
		}
	}
}

#endif

/* USER CODE END 4 */

/**
 * @brief  This function is executed in case of error occurrence.
 * @param  None
 * @retval None
 */
void Error_Handler(void)
{
	/* USER CODE BEGIN Error_Handler */
	/* User can add his own implementation to report the HAL error return state */
	while (1)
	{
	}
	/* USER CODE END Error_Handler */
}

#ifdef USE_FULL_ASSERT

/**
 * @brief Reports the name of the source file and the source line number
 * where the assert_param error has occurred.
 * @param file: pointer to the source file name
 * @param line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t* file, uint32_t line)
{
	/* USER CODE BEGIN 6 */
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
	/* USER CODE END 6 */

}

#endif

/**
 * @}
 */

/**
 * @}
 */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
